using System;
using System.Web.Compilation;
using System.Globalization;
using System.ComponentModel;
using System.Web.UI;
using System.CodeDom;
using System.Web;
using System.Threading;
using System.Diagnostics;

namespace Infrastructure.CrossCutting.Resource
{

    /// <summary>
    /// Custom expression builder support for $ExternalResources expressions.
    /// </summary>
    public class ExternalResourceExpressionBuilder : ExpressionBuilder
    {
        private static ResourceProviderFactory _resourceProviderFactory;

        public ExternalResourceExpressionBuilder()
        {
            Debug.WriteLine("ExternalResourceExpressionBuilder");

        }


        public static object GetGlobalResourceObject(string classKey, string resourceKey)
        {
            Debug.WriteLine(String.Format(CultureInfo.InvariantCulture, "ExternalResourceExpressionBuilder.GetGlobalResourceObject({0}, {1})", classKey, resourceKey));

            return ExternalResourceExpressionBuilder.GetGlobalResourceObject(classKey, resourceKey, null);
        }

        public static object GetGlobalResourceObject(string classKey, string resourceKey, CultureInfo culture)
        {
            Debug.WriteLine(String.Format(CultureInfo.InvariantCulture, "ExternalResourceExpressionBuilder.GetGlobalResourceObject({0}, {1}, {2})", classKey, resourceKey, culture));

            ExternalResourceExpressionBuilder.EnsureResourceProviderFactory();
            IResourceProvider provider = ExternalResourceExpressionBuilder._resourceProviderFactory.CreateGlobalResourceProvider(classKey);
            return provider.GetObject(resourceKey, culture);
        }

        public override object EvaluateExpression(object target, BoundPropertyEntry entry, object parsedData, ExpressionBuilderContext context)
        {
            Debug.WriteLine(String.Format(CultureInfo.InvariantCulture, "ExternalResourceExpressionBuilder.EvaluateExpression({0}, {1}, {2}, {3})", target, entry, parsedData, context));

            var fields = parsedData as ExternalResourceExpressionFields;

            ExternalResourceExpressionBuilder.EnsureResourceProviderFactory();
            IResourceProvider provider = ExternalResourceExpressionBuilder._resourceProviderFactory.CreateGlobalResourceProvider(fields.ClassKey);

            return provider.GetObject(fields.ResourceKey, null);
        }

        public override System.CodeDom.CodeExpression GetCodeExpression(BoundPropertyEntry entry, object parsedData, ExpressionBuilderContext context)
        {
            Debug.WriteLine(String.Format(CultureInfo.InvariantCulture, "ExternalResourceExpressionBuilder.GetCodeExpression({0}, {1}, {2})", entry, parsedData, context));

            var fields = parsedData as ExternalResourceExpressionFields;

            var exp = new CodeMethodInvokeExpression(new CodeTypeReferenceExpression(typeof(ExternalResourceExpressionBuilder)), "GetGlobalResourceObject", new CodePrimitiveExpression(fields.ClassKey), new CodePrimitiveExpression(fields.ResourceKey));

            return exp;
        }

        public override object ParseExpression(string expression, Type propertyType, ExpressionBuilderContext context)
        {
            Debug.WriteLine(String.Format(CultureInfo.InvariantCulture, "ExternalResourceExpressionBuilder.ParseExpression({0}, {1}, {2})", expression, propertyType, context));
            
            if (string.IsNullOrEmpty(expression))
            {
                throw new ArgumentException(expression);
            }

            ExternalResourceExpressionFields fields = null;
            string classKey = null;
            string resourceKey = null;
            
            string[] expParams = expression.Split(new char[] { ',' });
            if (expParams.Length > 2)
            {
                throw new ArgumentException(expression);
            }
            if (expParams.Length == 1)
            {
                throw new ArgumentException(expression);
            }
            else
            {
                classKey = expParams[0].Trim();
                resourceKey = expParams[1].Trim();
            }

            fields = new ExternalResourceExpressionFields(classKey, resourceKey);

            ExternalResourceExpressionBuilder.EnsureResourceProviderFactory();
            IResourceProvider rp = ExternalResourceExpressionBuilder._resourceProviderFactory.CreateGlobalResourceProvider(fields.ClassKey);
            
            object res = rp.GetObject(fields.ResourceKey, CultureInfo.InvariantCulture);
            if (res == null)
            {
                throw new ArgumentException(fields.ResourceKey);
            }
            return fields;
        }

        private static void EnsureResourceProviderFactory()
        {
            if (ExternalResourceExpressionBuilder._resourceProviderFactory == null)
            {
                ExternalResourceExpressionBuilder._resourceProviderFactory = new ExternalResourceProviderFactory();
            }
        }

        public override bool SupportsEvaluate
        {
            get
            {
                Debug.WriteLine("ExternalResourceExpressionBuilder.SupportsEvaluate");
                return true;
            }
        }
    }

    public class ExternalResourceExpressionFields
    {
        internal ExternalResourceExpressionFields(string classKey, string resourceKey)
        {
            this._classKey = classKey;
            this._resourceKey = resourceKey;
        }

        public string ClassKey
        {
            get
            {
                return this._classKey;
            }
        }

        public string ResourceKey
        {
            get
            {
                return this._resourceKey;
            }
        }

        private string _classKey;
        private string _resourceKey;


    }


}