
-- --------------------------------------------------
-- Entity Designer DDL Script for SQL Server 2005, 2008, and Azure
-- --------------------------------------------------
-- Date Created: 07/19/2012 15:00:35
-- Generated from EDMX file: D:\MyTFS\Framework\Infrastructure.Data.MainModule\MainModule\MainModuleDataModel.edmx
-- --------------------------------------------------

SET QUOTED_IDENTIFIER OFF;
GO
USE [NerdDinner];
GO
IF SCHEMA_ID(N'dbo') IS NULL EXECUTE(N'CREATE SCHEMA [dbo]');
GO

-- --------------------------------------------------
-- Dropping existing FOREIGN KEY constraints
-- --------------------------------------------------

IF OBJECT_ID(N'[dbo].[FK_RSVPs_Dinners]', 'F') IS NOT NULL
    ALTER TABLE [dbo].[RSVPs] DROP CONSTRAINT [FK_RSVPs_Dinners];
GO

-- --------------------------------------------------
-- Dropping existing tables
-- --------------------------------------------------

IF OBJECT_ID(N'[dbo].[Customers]', 'U') IS NOT NULL
    DROP TABLE [dbo].[Customers];
GO
IF OBJECT_ID(N'[dbo].[Dinners]', 'U') IS NOT NULL
    DROP TABLE [dbo].[Dinners];
GO
IF OBJECT_ID(N'[dbo].[RSVPs]', 'U') IS NOT NULL
    DROP TABLE [dbo].[RSVPs];
GO

-- --------------------------------------------------
-- Creating all tables
-- --------------------------------------------------

-- Creating table 'Customers'
CREATE TABLE [dbo].[Customers] (
    [CustomerId] int IDENTITY(1,1) NOT NULL,
    [AuthorizationId] nvarchar(255)  NOT NULL,
    [CustomerName] nvarchar(50)  NOT NULL,
    [CustomerEmail] nvarchar(50)  NULL,
    [CustomerPassword] nvarchar(50)  NULL,
    [IsRegister] bit  NOT NULL,
    [IsActive] bit  NOT NULL
);
GO

-- Creating table 'Blurbs'
CREATE TABLE [dbo].[Blurbs] (
    [BlurbId] int IDENTITY(1,1) NOT NULL,
    [ContentText] varchar(max)  NOT NULL
);
GO

-- Creating table 'Translations'
CREATE TABLE [dbo].[Translations] (
    [TranslateId] int IDENTITY(1,1) NOT NULL,
    [BlurbId] int  NOT NULL,
    [ResourceCode] nvarchar(20)  NULL,
    [CultureCode] nvarchar(10)  NOT NULL,
    [TranslateContent] varchar(max)  NULL
);
GO

-- Creating table 'Dinners'
CREATE TABLE [dbo].[Dinners] (
    [DinnerID] int IDENTITY(1,1) NOT NULL,
    [Title] nvarchar(50)  NOT NULL,
    [EventDate] datetime  NOT NULL,
    [Description] nvarchar(256)  NOT NULL,
    [HostedById] nvarchar(256)  NULL,
    [HostedBy] nvarchar(256)  NOT NULL,
    [ContactPhone] nvarchar(20)  NOT NULL,
    [Address] nvarchar(50)  NOT NULL,
    [Country] nvarchar(30)  NOT NULL,
    [Latitude] float  NOT NULL,
    [Longitude] float  NOT NULL
);
GO

-- Creating table 'RSVPs'
CREATE TABLE [dbo].[RSVPs] (
    [RsvpID] int  NOT NULL,
    [DinnerID] int IDENTITY(1,1) NOT NULL,
    [AttendeeName] nvarchar(256)  NOT NULL,
    [AttendeeNameId] nvarchar(256)  NULL
);
GO

-- --------------------------------------------------
-- Creating all PRIMARY KEY constraints
-- --------------------------------------------------

-- Creating primary key on [CustomerId] in table 'Customers'
ALTER TABLE [dbo].[Customers]
ADD CONSTRAINT [PK_Customers]
    PRIMARY KEY CLUSTERED ([CustomerId] ASC);
GO

-- Creating primary key on [BlurbId] in table 'Blurbs'
ALTER TABLE [dbo].[Blurbs]
ADD CONSTRAINT [PK_Blurbs]
    PRIMARY KEY CLUSTERED ([BlurbId] ASC);
GO

-- Creating primary key on [TranslateId] in table 'Translations'
ALTER TABLE [dbo].[Translations]
ADD CONSTRAINT [PK_Translations]
    PRIMARY KEY CLUSTERED ([TranslateId] ASC);
GO

-- Creating primary key on [DinnerID] in table 'Dinners'
ALTER TABLE [dbo].[Dinners]
ADD CONSTRAINT [PK_Dinners]
    PRIMARY KEY CLUSTERED ([DinnerID] ASC);
GO

-- Creating primary key on [RsvpID] in table 'RSVPs'
ALTER TABLE [dbo].[RSVPs]
ADD CONSTRAINT [PK_RSVPs]
    PRIMARY KEY CLUSTERED ([RsvpID] ASC);
GO

-- --------------------------------------------------
-- Creating all FOREIGN KEY constraints
-- --------------------------------------------------

-- Creating foreign key on [BlurbId] in table 'Translations'
ALTER TABLE [dbo].[Translations]
ADD CONSTRAINT [FK_Translations_Blurbs]
    FOREIGN KEY ([BlurbId])
    REFERENCES [dbo].[Blurbs]
        ([BlurbId])
    ON DELETE NO ACTION ON UPDATE NO ACTION;

-- Creating non-clustered index for FOREIGN KEY 'FK_Translations_Blurbs'
CREATE INDEX [IX_FK_Translations_Blurbs]
ON [dbo].[Translations]
    ([BlurbId]);
GO

-- Creating foreign key on [DinnerID] in table 'RSVPs'
ALTER TABLE [dbo].[RSVPs]
ADD CONSTRAINT [FK_RSVPs_Dinners]
    FOREIGN KEY ([DinnerID])
    REFERENCES [dbo].[Dinners]
        ([DinnerID])
    ON DELETE NO ACTION ON UPDATE NO ACTION;

-- Creating non-clustered index for FOREIGN KEY 'FK_RSVPs_Dinners'
CREATE INDEX [IX_FK_RSVPs_Dinners]
ON [dbo].[RSVPs]
    ([DinnerID]);
GO

-- --------------------------------------------------
-- Script has ended
-- --------------------------------------------------