﻿#region

using System;
using System.Web.Mvc;

#endregion

namespace Presentation.Web.Core.Breadcrumbs
{
    public class BreadcrumbAttribute : ActionFilterAttribute
    {
        private string breadcrumbItemTitle;

        public BreadcrumbAttribute(string title, int level)
        {
            Title = title;
            Level = level;
        }

        public string Title { get; set; }
        public int Level { get; set; }

        public override void OnResultExecuting(ResultExecutingContext filterContext)
        {
            var controller = (Controller) filterContext.Controller;

            string breadcrumbText = controller.GetBreadcrumbText();

            if (!String.IsNullOrEmpty(Title))
            {
                breadcrumbItemTitle = String.Format(Title, breadcrumbText);
            }

            Breadcrumb[] breadcrumbs = getBreadcrumbManager(filterContext)
                .PushBreadcrumb(
                    filterContext.HttpContext.Request.RawUrl,
                    breadcrumbItemTitle,
                    Level);

            controller.ViewData.Add("breadcrumbs", breadcrumbs);

            base.OnResultExecuting(filterContext);
        }

        private static IBreadcrumbManager getBreadcrumbManager(ResultExecutingContext filterContext)
        {
            var bcm = filterContext.HttpContext.Session["breadcrumbManager"] as IBreadcrumbManager;
            if (bcm == null)
            {
                bcm = new BreadcrumbManager();
                filterContext.HttpContext.Session["breadcrumbManager"] = bcm;
            }
            return bcm;
        }
    }
}