﻿using System;
using System.Linq;
using System.Web.Mvc;
using Application.MainModule.DinnerManagement;
using System.ComponentModel;
using Presentation.Web.UI.ActionResults;
using Presentation.Web.UI.Attibutes;
using Presentation.Web.ViewModels;
using Presentation.Web.Core.Geolocation;

namespace Presentation.Web.UI.Controllers
{
    [HandleErrorWithELMAH]
    public class ServiceController : Controller
    {
        readonly IDinnerManagementService _dinnerService;

        public ServiceController(IDinnerManagementService dinnerService)
        {
            _dinnerService = dinnerService;
        }

        [OutputCache(VaryByParam = "none", Duration = 300)]
        public ActionResult RSS()
        {
            var dinners = _dinnerService.FindUpcomingDinners();

            if (dinners == null)
                return View("NotFound");

            return new RssResult(dinners.ToList(), "Upcoming Nerd Dinners");
        }

        [OutputCache(VaryByParam = "none", Duration = 300)]
        public ActionResult iCalFeed()
        {
            var dinners = _dinnerService.FindUpcomingDinners();
            if (dinners == null)
                return View("NotFound");
            return new iCalResult(dinners.ToList(), "NerdDinners.ics");
        }

        public ActionResult iCal(int id)
        {
            var dinner = _dinnerService.FindDinner(id);
            if (dinner == null)
                return View("NotFound");
            return new iCalResult(dinner, "NerdDinner.ics");
        }

        public ActionResult Flair([DefaultValue("html")]string format)
        {
            string SourceIP = string.IsNullOrEmpty(Request.ServerVariables["HTTP_X_FORWARDED_FOR"]) ?
                Request.ServerVariables["REMOTE_ADDR"] :
                Request.ServerVariables["HTTP_X_FORWARDED_FOR"];

            var location = GeolocationService.HostIpToPlaceName(SourceIP);
            var dinners = _dinnerService.
                FindByLocation(location.Position.Lat, location.Position.Long).
                OrderByDescending(p => p.EventDate).Take(3);

            // Select the view we'll return. Using a switch because we'll add in JSON and other formats later.
            // Will probably extract or refactor this.
            string view;
            switch (format.ToLower())
            {
                case "javascript":
                    view = "JavascriptFlair";
                    break;
                default:
                    view = "Flair";
                    break;
            }

            return View(
                view,
                new FlairViewModel
                {
                    Dinners = dinners.ToList(),
                    LocationName = string.IsNullOrEmpty(location.City) ? "you" : String.Format("{0}, {1}", location.City, location.RegionName)
                }
            );
        }
    }
}
