﻿using System;
using System.Collections.Generic;
using System.Data.Services;
using System.Data.Services.Common;
using System.Linq;
using System.ServiceModel;
using System.ServiceModel.Web;
using System.Web;
using Application.MainModule.DinnerManagement;
using Domain.MainModule.Entities;
using Infrastructure.CrossCutting.IoC;
using Infrastructure.Data.MainModule.MainModule;
using Presentation.Web.Core.Geolocation;
using Presentation.Web.UI.Attibutes;

namespace Presentation.Web.UI.Service
{
    [ServiceBehavior(IncludeExceptionDetailInFaults = true, InstanceContextMode = InstanceContextMode.Single, ConcurrencyMode = System.ServiceModel.ConcurrencyMode.Single)]
    [JSONPSupportBehavior]
    public class ODataServices : DataService<MainModuleContext>
    {
        private readonly IDinnerManagementService _dinnerManagementService; 
        //
        // Dependency Injection enabled constructors

        public ODataServices(){
            _dinnerManagementService = _dinnerManagementService = IoCFactory.CurrentContainer.Resolve<IDinnerManagementService>();
             
        }

        public ODataServices(IDinnerManagementService dinnerManagementService)
        {
            _dinnerManagementService = dinnerManagementService;
        }

        protected override MainModuleContext CreateDataSource()
        {
            var nd = base.CreateDataSource();
            nd.Configuration.ProxyCreationEnabled = false;
            return nd;
        }

        // This method is called only once to initialize service-wide policies.
        public static void InitializeService(DataServiceConfiguration config)
        {
            //We have thousands of rows so setup server-page
            config.SetEntitySetPageSize("*", 100);

            // We're exposing both Dinners and RSVPs for read
            config.SetEntitySetAccessRule("Dinners", EntitySetRights.AllRead);
            config.SetEntitySetAccessRule("RSVPs", EntitySetRights.AllRead);
                      config.SetServiceOperationAccessRule("DinnersNearMe", ServiceOperationRights.AllRead);
            config.SetServiceOperationAccessRule("*", ServiceOperationRights.All);
            config.DataServiceBehavior.MaxProtocolVersion = DataServiceProtocolVersion.V2;
            #if DEBUG
            config.UseVerboseErrors = true;
            #endif
        }

        protected override void OnStartProcessingRequest(ProcessRequestArgs args)
        {
            base.OnStartProcessingRequest(args);

            HttpContext context = HttpContext.Current;
            if (null != context)
            {
                HttpCachePolicy c = context.Response.Cache;
                c.SetCacheability(HttpCacheability.ServerAndPrivate);
                c.SetExpires(context.Timestamp.AddSeconds(30));
                c.VaryByHeaders["Accept"] = true;
                c.VaryByHeaders["Accept-Charset"] = true;
                c.VaryByHeaders["Accept-Encoding"] = true;
                c.VaryByParams["*"] = true;
            }
        }

        [WebGet]
        public IEnumerable<Dinners> FindUpcomingDinners()
        {
            return _dinnerManagementService.FindUpcomingDinners();
        }

        // http://localhost:60848/Services.svc/DinnersNearMe?placeOrZip='12345'
        [WebGet]
        public IEnumerable<Dinners> DinnersNearMe(string placeOrZip)
        {
            if (String.IsNullOrEmpty(placeOrZip)) return null; ;

            LatLong location = GeolocationService.PlaceOrZipToLatLong(placeOrZip);

            var dinners = _dinnerManagementService.
                            FindByLocation(location.Lat, location.Long).
                            OrderByDescending(p => p.EventDate);
            return dinners;
        }

    }
}
